@@ -43,14 +43,15 @@
 
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-ontology.h>
-#include <libtracker-common/tracker-statement-list.h>
 #include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-utils.h>
 
+#include <libtracker-extract/tracker-extract.h>
+#include <libtracker-extract/tracker-xmp.h>
+#include <libtracker-extract/tracker-iptc.h>
+#include <libtracker-extract/tracker-exif.h>
+
 #include "tracker-main.h"
-#include "tracker-xmp.h"
-#include "tracker-iptc.h"
-#include "tracker-exif.h"
 
 #ifdef HAVE_LIBEXIF
 #define EXIF_NAMESPACE          "Exif"
@@ -85,9 +86,14 @@ typedef struct {
 	gchar *metering_mode;
 	gchar *creator;
 	gchar *comment;
+	gchar *city;
+	gchar *state;
+	gchar *address;
+	gchar *country; 
 } MergeData;
 
 static void extract_jpeg (const gchar          *filename,
+                          TrackerSparqlBuilder *preupdate,
                           TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
@@ -147,6 +153,7 @@ extract_jpeg_error_exit (j_common_ptr cinfo)
 
 static void
 extract_jpeg (const gchar          *uri,
+              TrackerSparqlBuilder *preupdate,
               TrackerSparqlBuilder *metadata)
 {
 	struct jpeg_decompress_struct cinfo;
@@ -174,7 +181,6 @@ extract_jpeg (const gchar          *uri,
 		MergeData md = { 0 };
 		gchar *comment = NULL;
 
-		tracker_sparql_builder_subject_iri (metadata, uri);
 		tracker_sparql_builder_predicate (metadata, "a");
 		tracker_sparql_builder_object (metadata, "nfo:Image");
 		tracker_sparql_builder_predicate (metadata, "a");
@@ -212,7 +218,7 @@ extract_jpeg (const gchar          *uri,
 			gsize len;
 #ifdef HAVE_LIBIPTCDATA
 			gsize offset;
-			gsize sublen;
+			guint sublen;
 #endif /* HAVE_LIBIPTCDATA */
 
 			switch (marker->marker) {
@@ -227,7 +233,7 @@ extract_jpeg (const gchar          *uri,
 
 #ifdef HAVE_LIBEXIF
 				if (strncmp (EXIF_NAMESPACE, str, EXIF_NAMESPACE_LENGTH) == 0) {
-					tracker_read_exif ((unsigned char*) marker->data,
+					tracker_exif_read ((unsigned char*) marker->data,
 					                   len,
 					                   uri,
 					                   &ed);
@@ -236,7 +242,7 @@ extract_jpeg (const gchar          *uri,
 
 #ifdef HAVE_EXEMPI
 				if (strncmp (XMP_NAMESPACE, str, XMP_NAMESPACE_LENGTH) == 0) {
-					tracker_read_xmp (str + XMP_NAMESPACE_LENGTH,
+					tracker_xmp_read (str + XMP_NAMESPACE_LENGTH,
 					                  len - XMP_NAMESPACE_LENGTH,
 					                  uri,
 					                  &xd);
@@ -252,7 +258,7 @@ extract_jpeg (const gchar          *uri,
 				if (strncmp (PS3_NAMESPACE, str, PS3_NAMESPACE_LENGTH) == 0) {
 					offset = iptc_jpeg_ps3_find_iptc (str, len, &sublen);
 					if (offset > 0) {
-						tracker_read_iptc (str + offset,
+						tracker_iptc_read (str + offset,
 						                   sublen,
 						                   uri,
 						                   &id);
@@ -271,13 +277,13 @@ extract_jpeg (const gchar          *uri,
 		}
 
 		/* Don't merge if the make is in the model */
-		if ((xd.Make == NULL || xd.Model == NULL) ||
-		    (xd.Make && xd.Model && strstr (xd.Model, xd.Make) == NULL)) {
-			md.camera = tracker_merge (" ", 2, xd.Make, xd.Model);
+		if ((xd.make == NULL || xd.model == NULL) ||
+		    (xd.make && xd.model && strstr (xd.model, xd.make) == NULL)) {
+			md.camera = tracker_merge (" ", 2, xd.make, xd.model);
 		} else {
-			md.camera = g_strdup (xd.Model);
-			g_free (xd.Model);
-			g_free (xd.Make);
+			md.camera = g_strdup (xd.model);
+			g_free (xd.model);
+			g_free (xd.make);
 		}
 
 		if (!md.camera) {
@@ -294,20 +300,26 @@ extract_jpeg (const gchar          *uri,
 			g_free (ed.make);
 		}
 
-		md.title = tracker_coalesce (3, xd.title, xd.Title, ed.document_name, xd.PDFtitle);
-		md.orientation = tracker_coalesce (3, ed.orientation, xd.Orientation, id.image_orientation);
-		md.copyright = tracker_coalesce (3, ed.copyright, xd.Copyright, xd.rights, id.copyright_notice);
-		md.white_balance = tracker_coalesce (2, ed.white_balance, xd.WhiteBalance);
-		md.fnumber =  tracker_coalesce (2, ed.fnumber, xd.FNumber);
-		md.flash =  tracker_coalesce (2, ed.flash, xd.Flash);
-		md.focal_length =  tracker_coalesce (2, ed.focal_length, xd.FocalLength);
-		md.artist =  tracker_coalesce (3, ed.artist, xd.Artist, xd.contributor);
-		md.exposure_time =  tracker_coalesce (2, ed.exposure_time, xd.ExposureTime);
-		md.iso_speed_ratings =  tracker_coalesce (2, ed.iso_speed_ratings, xd.ISOSpeedRatings);
-		md.date =  tracker_coalesce (5, ed.time, xd.date, id.date_created, ed.time_original, xd.DateTimeOriginal);
-		md.description = tracker_coalesce (2, ed.description, xd.description);
-		md.metering_mode =  tracker_coalesce (2, ed.metering_mode, xd.MeteringMode);
-		md.creator =  tracker_coalesce (3, id.byline, xd.creator, id.credit);
+		md.title = tracker_coalesce (4, xd.title, ed.document_name, xd.title2, xd.pdf_title);
+		md.orientation = tracker_coalesce (3, xd.orientation, ed.orientation, id.image_orientation);
+		md.copyright = tracker_coalesce (4, xd.copyright, xd.rights, ed.copyright, id.copyright_notice);
+		md.white_balance = tracker_coalesce (2, xd.white_balance, ed.white_balance);
+		md.fnumber =  tracker_coalesce (2, xd.fnumber, ed.fnumber);
+		md.flash =  tracker_coalesce (2, xd.flash, ed.flash);
+		md.focal_length =  tracker_coalesce (2, xd.focal_length, ed.focal_length);
+		md.artist =  tracker_coalesce (3, xd.artist, ed.artist, xd.contributor);
+		md.exposure_time =  tracker_coalesce (2, xd.exposure_time, ed.exposure_time);
+		md.iso_speed_ratings =  tracker_coalesce (2, xd.iso_speed_ratings, ed.iso_speed_ratings);
+		md.date =  tracker_coalesce (5, xd.date, xd.time_original, ed.time, id.date_created, ed.time_original);
+		md.description = tracker_coalesce (2, xd.description, ed.description);
+		md.metering_mode = tracker_coalesce (2, xd.metering_mode, ed.metering_mode);
+
+		md.city = tracker_coalesce (2, xd.city, id.city);
+		md.state = tracker_coalesce (2, xd.state, id.state);
+		md.address = tracker_coalesce (2, xd.address, id.sublocation);
+		md.country  = tracker_coalesce (2, xd.country, id.country_name);
+
+		md.creator =  tracker_coalesce (3, xd.creator, id.byline, id.credit);
 		md.comment = tracker_coalesce (2, comment, ed.user_comment);
 
 		/* Prioritize on native dimention in all cases */
@@ -315,18 +327,32 @@ extract_jpeg (const gchar          *uri,
 		tracker_sparql_builder_object_int64 (metadata, cinfo.image_width);
 		g_free (ed.x_dimension);
 
+		/* TODO: add ontology and store this */
+		g_free (ed.software);
+
 		tracker_sparql_builder_predicate (metadata, "nfo:height");
 		tracker_sparql_builder_object_int64 (metadata, cinfo.image_height);
 		g_free (ed.y_dimension);
 
+		if (id.contact) {
+			tracker_sparql_builder_predicate (metadata, "nco:representative");
+			tracker_sparql_builder_object_blank_open (metadata);
+			tracker_sparql_builder_predicate (metadata, "a");
+			tracker_sparql_builder_object (metadata, "nco:Contact");
+			tracker_sparql_builder_predicate (metadata, "nco:fullname");
+			tracker_sparql_builder_object_unvalidated (metadata, id.contact);
+			tracker_sparql_builder_object_blank_close (metadata);
+			g_free (id.contact);
+		}
+
 		if (xd.keywords) {
 			insert_keywords (metadata, xd.keywords);
 			g_free (xd.keywords);
 		}
 
-		if (xd.PDFkeywords) {
-			insert_keywords (metadata, xd.PDFkeywords);
-			g_free (xd.PDFkeywords);
+		if (xd.pdf_keywords) {
+			insert_keywords (metadata, xd.pdf_keywords);
+			g_free (xd.pdf_keywords);
 		}
 
 		if (xd.subject) {
@@ -351,6 +377,12 @@ extract_jpeg (const gchar          *uri,
 			g_free (xd.type);
 		}
 
+		if (xd.rating) {
+			tracker_sparql_builder_predicate (metadata, "nao:numericRating");
+			tracker_sparql_builder_object_unvalidated (metadata, xd.rating);
+			g_free (xd.rating);
+		}
+
 		if (xd.format) {
 			tracker_sparql_builder_predicate (metadata, "dc:format");
 			tracker_sparql_builder_object_unvalidated (metadata, xd.format);
@@ -500,51 +532,75 @@ extract_jpeg (const gchar          *uri,
 		}
 
 		if (md.creator) {
+
+			if (id.byline_title) {
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+
+				tracker_sparql_builder_subject (preupdate, "_:affiliation_by_line");
+				tracker_sparql_builder_predicate (preupdate, "a");
+				tracker_sparql_builder_object (preupdate, "nco:Affiliation");
+
+				tracker_sparql_builder_predicate (preupdate, "nco:title");
+				tracker_sparql_builder_object_unvalidated (preupdate, id.byline_title);
+
+				tracker_sparql_builder_insert_close (preupdate);
+			}
+
 			tracker_sparql_builder_predicate (metadata, "nco:creator");
 			tracker_sparql_builder_object_blank_open (metadata);
 			tracker_sparql_builder_predicate (metadata, "a");
 			tracker_sparql_builder_object (metadata, "nco:Contact");
 			tracker_sparql_builder_predicate (metadata, "nco:fullname");
 			tracker_sparql_builder_object_unvalidated (metadata, md.creator);
+
+			if (id.byline_title) {
+				tracker_sparql_builder_predicate (metadata, "a");
+				tracker_sparql_builder_object (metadata, "nco:PersonContact");
+				tracker_sparql_builder_predicate (metadata, "nco:hasAffiliation");
+				tracker_sparql_builder_object (metadata, "_:affiliation_by_line");
+			}
+
 			tracker_sparql_builder_object_blank_close (metadata);
 			g_free (md.creator);
 		}
 
+		g_free (id.byline_title);
+
 		if (md.comment) {
 			tracker_sparql_builder_predicate (metadata, "nie:comment");
 			tracker_sparql_builder_object_unvalidated (metadata, md.comment);
 			g_free (md.comment);
 		}
 
-		if (xd.Address || xd.Country || xd.City) {
+		if (md.city || md.state || md.address || md.country) {
 			tracker_sparql_builder_predicate (metadata, "mlo:location");
-	
+
 			tracker_sparql_builder_object_blank_open (metadata);
 			tracker_sparql_builder_predicate (metadata, "a");
 			tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
 	
-			if (xd.Address) {
+			if (md.address) {
 				tracker_sparql_builder_predicate (metadata, "mlo:address");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.Address);
-				g_free (xd.Address);
+				tracker_sparql_builder_object_unvalidated (metadata, md.address);
+				g_free (md.address);
 			}
 	
-			if (xd.State) {
+			if (md.state) {
 				tracker_sparql_builder_predicate (metadata, "mlo:state");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.State);
-				g_free (xd.State);
+				tracker_sparql_builder_object_unvalidated (metadata, md.state);
+				g_free (md.state);
 			}
 	
-			if (xd.City) {
+			if (md.city) {
 				tracker_sparql_builder_predicate (metadata, "mlo:city");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.City);
-				g_free (xd.City);
+				tracker_sparql_builder_object_unvalidated (metadata, md.city);
+				g_free (md.city);
 			}
 	
-			if (xd.Country) {
+			if (md.country) {
 				tracker_sparql_builder_predicate (metadata, "mlo:country");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.Country);
-				g_free (xd.Country);
+				tracker_sparql_builder_object_unvalidated (metadata, md.country);
+				g_free (md.country);
 			}
 		
 			tracker_sparql_builder_object_blank_close (metadata);
@@ -559,7 +615,7 @@ extract_jpeg (const gchar          *uri,
 }
 
 TrackerExtractData *
-tracker_get_extract_data (void)
+tracker_extract_get_data (void)
 {
 	return data;
 }
