@@ -83,8 +83,10 @@
 #define SLIDELISTWITHTEXT_RECORD_TYPE           0x0FF0
 
 static void extract_msoffice   (const gchar          *uri,
+                                TrackerSparqlBuilder *preupdate,
                                 TrackerSparqlBuilder *metadata);
 static void extract_powerpoint (const gchar          *uri,
+                                TrackerSparqlBuilder *preupdate,
                                 TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
@@ -784,11 +786,8 @@ extract_msword_content (GsfInfile *infile,
 		}
 	}
 
-	g_free (clx);
-
 	/* iterate over pieces and save text to the content -variable */
 	for (i = 0; i < piece_count; i++) {
-
 		/* logical position of the text piece in the document_stream */
 		piece_start = read_32bit (piece_table+(i*4));
 		piece_end = read_32bit (piece_table+((i+1)*4));
@@ -859,6 +858,7 @@ extract_msword_content (GsfInfile *infile,
 
 	g_object_unref (document_stream);
 	g_object_unref (table_stream);
+	g_free (clx);
 
 	if (content) {
 		normalized = tracker_text_normalize (content->str, n_words, NULL);
@@ -894,7 +894,6 @@ extract_summary (TrackerSparqlBuilder *metadata,
 	gchar    *content;
 	gboolean  is_encrypted = FALSE;
 
-	tracker_sparql_builder_subject_iri (metadata, uri);
 	tracker_sparql_builder_predicate (metadata, "a");
 	tracker_sparql_builder_object (metadata, "nfo:PaginatedTextDocument");
 
@@ -902,11 +901,14 @@ extract_summary (TrackerSparqlBuilder *metadata,
 
 	if (stream) {
 		GsfDocMetaData *md;
+		GError *err = NULL;
 		ForeachInfo     info = { metadata, uri };
 
 		md = gsf_doc_meta_data_new ();
+		err = gsf_msole_metadata_read (stream, md);
 
-		if (gsf_msole_metadata_read (stream, md)) {
+		if (err) {
+			g_error_free (err);
 			g_object_unref (md);
 			g_object_unref (stream);
 			gsf_shutdown ();
@@ -923,11 +925,14 @@ extract_summary (TrackerSparqlBuilder *metadata,
 
 	if (stream) {
 		GsfDocMetaData *md;
+		GError *err = NULL;
 		ForeachInfo     info = { metadata, uri };
 
 		md = gsf_doc_meta_data_new ();
 
-		if (gsf_msole_metadata_read (stream, md)) {
+		err = gsf_msole_metadata_read (stream, md);
+		if (err) {
+			g_error_free (err);
 			g_object_unref (md);
 			g_object_unref (stream);
 			gsf_shutdown ();
@@ -1002,6 +1007,7 @@ open_uri (const gchar *uri)
  */
 static void
 extract_msoffice (const gchar          *uri,
+                  TrackerSparqlBuilder *preupdate,
                   TrackerSparqlBuilder *metadata)
 {
 	GsfInfile *infile = open_uri(uri);
@@ -1019,6 +1025,7 @@ extract_msoffice (const gchar          *uri,
  */
 static void
 extract_powerpoint (const gchar          *uri,
+                    TrackerSparqlBuilder *preupdate,
                     TrackerSparqlBuilder *metadata)
 {
 	GsfInfile *infile = open_uri(uri);
