@@ -41,8 +41,9 @@
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-utils.h>
 
-#include "tracker-main.h"
-#include "tracker-xmp.h"
+#include <libtracker-extract/tracker-extract.h>
+#include <libtracker-extract/tracker-xmp.h>
+#include <libtracker-extract/tracker-exif.h>
 
 #define RFC1123_DATE_FORMAT "%d %B %Y %H:%M:%S %z"
 
@@ -56,6 +57,21 @@
 
 typedef struct {
 	gchar *title, *copyright, *creator, *description, *date, *license;
+	gchar *artist;
+	gchar *camera;
+	gchar *orientation;
+	gchar *white_balance;
+	gchar *fnumber;
+	gchar *flash;
+	gchar *focal_length;
+	gchar *exposure_time;
+	gchar *iso_speed_ratings;
+	gchar *metering_mode;
+	gchar *comment;
+	gchar *city;
+	gchar *state;
+	gchar *address;
+	gchar *country;
 } PngNeedsMergeData;
 
 typedef struct {
@@ -63,9 +79,10 @@ typedef struct {
 		*creation_time, *title, *disclaimer;
 } PngData;
 
-static gchar *rfc1123_to_iso8601_date (gchar       *rfc_date);
-static void   extract_png             (const gchar *filename,
-                                       TrackerSparqlBuilder   *metadata);
+static gchar *rfc1123_to_iso8601_date (gchar                *rfc_date);
+static void   extract_png             (const gchar          *filename,
+                                       TrackerSparqlBuilder *preupdate,
+                                       TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
 	{ "image/png", extract_png },
@@ -121,6 +138,7 @@ read_metadata (png_structp png_ptr, png_infop info_ptr, const gchar *uri, Tracke
 	png_textp    text_ptr;
 	PngNeedsMergeData merge_data = { 0 };
 	PngData png_data = { 0 };
+	TrackerExifData exif_data = { 0 };
 	TrackerXmpData xmp_data = { 0 };
 
 	if (png_get_text (png_ptr, info_ptr, &text_ptr, &num_text) > 0) {
@@ -141,7 +159,7 @@ read_metadata (png_structp png_ptr, png_infop info_ptr, const gchar *uri, Tracke
 				 * theoretically possible that the function gets
 				 * called multiple times */
 
-				tracker_read_xmp (text_ptr[i].text,
+				tracker_xmp_read (text_ptr[i].text,
 				                  text_ptr[i].itxt_length,
 				                  uri, &xmp_data);
 
@@ -149,6 +167,20 @@ read_metadata (png_structp png_ptr, png_infop info_ptr, const gchar *uri, Tracke
 			}
 #endif
 
+#if defined(HAVE_LIBEXIF) && defined(PNG_iTXt_SUPPORTED)
+
+			/* I'm not certain this is the key for EXIF. Using key according to
+			 * this document about exiftool:
+			 * http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/PNG.html#TextualData */
+
+			if (g_strcmp0 ("Raw profile type exif", text_ptr[i].key) == 0) {
+				tracker_exif_read (text_ptr[i].text,
+				                   text_ptr[i].itxt_length, 
+				                   uri, &exif_data);
+				continue;
+			}
+#endif /* HAVE_LIBEXIF */
+
 			if (g_strcmp0 (text_ptr[i].key, "Author") == 0) {
 				png_data.author = g_strdup (text_ptr[i].text);
 				continue;
@@ -190,31 +222,92 @@ read_metadata (png_structp png_ptr, png_infop info_ptr, const gchar *uri, Tracke
 			}
 		}
 
-		merge_data.creator = tracker_coalesce (3, png_data.creator,
-		                                       png_data.author,
-		                                       xmp_data.creator);
+		/* Don't merge if the make is in the model */
+		if ((xmp_data.make == NULL || xmp_data.model == NULL) ||
+		    (xmp_data.make && xmp_data.model && strstr (xmp_data.model, xmp_data.make) == NULL)) {
+			merge_data.camera = tracker_merge (" ", 2, xmp_data.make, xmp_data.model);
+		} else {
+			merge_data.camera = g_strdup (xmp_data.model);
+			g_free (xmp_data.model);
+			g_free (xmp_data.make);
+		}
 
-		merge_data.title = tracker_coalesce (2, png_data.title,
-		                                     xmp_data.title, xmp_data.Title);
+		if (!merge_data.camera) {
+			if ((exif_data.make == NULL || exif_data.model == NULL) ||
+			    (exif_data.make && exif_data.model && strstr (exif_data.model, exif_data.make) == NULL)) {
+				merge_data.camera = tracker_merge (" ", 2, exif_data.make, exif_data.model);
+			} else {
+				merge_data.camera = g_strdup (exif_data.model);
+				g_free (exif_data.model);
+				g_free (exif_data.make);
+			}
+		} else {
+			g_free (exif_data.model);
+			g_free (exif_data.make);
+		}
 
-		merge_data.copyright = tracker_coalesce (2, png_data.copyright,
-		                                         xmp_data.rights);
+		merge_data.creator = tracker_coalesce (3, xmp_data.creator, 
+		                                       png_data.creator,
+		                                       png_data.author);
 
-		merge_data.license = tracker_coalesce (2, png_data.disclaimer,
-		                                       xmp_data.license);
+		merge_data.title = tracker_coalesce (5, xmp_data.title, 
+		                                     png_data.title,
+		                                     exif_data.document_name,
+		                                     xmp_data.title2,
+		                                     xmp_data.pdf_title);
 
-		merge_data.description = tracker_coalesce (2, png_data.description,
-		                                           xmp_data.description);
+		merge_data.copyright = tracker_coalesce (3, xmp_data.rights, 
+		                                         png_data.copyright,
+		                                         exif_data.copyright);
 
-		merge_data.date = tracker_coalesce (3, png_data.creation_time,
-		                                    xmp_data.date,
-		                                    xmp_data.DateTimeOriginal);
+		merge_data.license = tracker_coalesce (2, xmp_data.license, 
+		                                       png_data.disclaimer);
 
+		merge_data.description = tracker_coalesce (3, xmp_data.description,
+		                                           png_data.description,
+		                                           exif_data.description);
 
-		if (png_data.comment) {
+		merge_data.date = tracker_coalesce (5, xmp_data.date,
+		                                    xmp_data.time_original,
+		                                    png_data.creation_time,
+		                                    exif_data.time,
+		                                    exif_data.time_original);
+
+		merge_data.comment = tracker_coalesce (2, png_data.comment,
+		                                       exif_data.user_comment);
+
+		merge_data.artist = tracker_coalesce (3, xmp_data.artist,
+		                                      exif_data.artist,
+		                                      xmp_data.contributor);
+
+		merge_data.orientation = tracker_coalesce (2, xmp_data.orientation, 
+		                                           exif_data.orientation);
+
+		merge_data.exposure_time = tracker_coalesce (2, xmp_data.exposure_time, 
+		                                             exif_data.exposure_time);
+
+		merge_data.iso_speed_ratings = tracker_coalesce (2, xmp_data.iso_speed_ratings, 
+		                                                 exif_data.iso_speed_ratings);
+
+		merge_data.fnumber = tracker_coalesce (2, xmp_data.fnumber, 
+		                                       exif_data.fnumber);
+
+		merge_data.flash = tracker_coalesce (2, xmp_data.flash, 
+		                                     exif_data.flash);
+
+		merge_data.focal_length = tracker_coalesce (2, xmp_data.focal_length, 
+		                                            exif_data.focal_length);
+
+		merge_data.metering_mode = tracker_coalesce (2, xmp_data.metering_mode, 
+		                                             exif_data.metering_mode);
+
+		merge_data.white_balance = tracker_coalesce (2, xmp_data.white_balance, 
+		                                             exif_data.white_balance);
+
+		if (merge_data.comment) {
 			tracker_sparql_builder_predicate (metadata, "nie:comment");
-			tracker_sparql_builder_object_unvalidated (metadata, png_data.comment);
-			g_free (png_data.comment);
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.comment);
+			g_free (merge_data.comment);
 		}
 
 		if (merge_data.license) {
@@ -223,6 +316,13 @@ read_metadata (png_structp png_ptr, png_infop info_ptr, const gchar *uri, Tracke
 			g_free (merge_data.license);
 		}
 
+		/* TODO: add ontology and store this */
+		g_free (exif_data.software);
+
+		g_free (exif_data.x_dimension);
+		g_free (exif_data.y_dimension);
+		g_free (exif_data.image_width);
+
 		if (merge_data.creator) {
 			tracker_sparql_builder_predicate (metadata, "nco:creator");
 
@@ -260,12 +360,90 @@ read_metadata (png_structp png_ptr, png_infop info_ptr, const gchar *uri, Tracke
 			g_free (merge_data.title);
 		}
 
+		if (merge_data.camera) {
+			tracker_sparql_builder_predicate (metadata, "nmm:camera");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.camera);
+			g_free (merge_data.camera);
+		}
+
+		if (merge_data.artist) {
+			tracker_sparql_builder_predicate (metadata, "nco:contributor");
+
+			tracker_sparql_builder_object_blank_open (metadata);
+			tracker_sparql_builder_predicate (metadata, "a");
+			tracker_sparql_builder_object (metadata, "nco:Contact");
+
+			tracker_sparql_builder_predicate (metadata, "nco:fullname");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.artist);
+			tracker_sparql_builder_object_blank_close (metadata);
+			g_free (merge_data.artist);
+		}
+
+		if (merge_data.orientation) {
+			tracker_sparql_builder_predicate (metadata, "nfo:orientation");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.orientation);
+			g_free (merge_data.orientation);
+		}
+
+		if (merge_data.exposure_time) {
+			tracker_sparql_builder_predicate (metadata, "nmm:exposureTime");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.exposure_time);
+			g_free (merge_data.exposure_time);
+		}
+
+		if (merge_data.iso_speed_ratings) {
+			tracker_sparql_builder_predicate (metadata, "nmm:isoSpeed");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.iso_speed_ratings);
+			g_free (merge_data.iso_speed_ratings);
+		}
+
+		if (merge_data.white_balance) {
+			tracker_sparql_builder_predicate (metadata, "nmm:whiteBalance");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.white_balance);
+			g_free (merge_data.white_balance);
+		}
+
+		if (merge_data.fnumber) {
+			tracker_sparql_builder_predicate (metadata, "nmm:fnumber");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.fnumber);
+			g_free (merge_data.fnumber);
+		}
+
+		if (merge_data.flash) {
+			tracker_sparql_builder_predicate (metadata, "nmm:flash");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.flash);
+			g_free (merge_data.flash);
+		}
+
+		if (merge_data.focal_length) {
+			tracker_sparql_builder_predicate (metadata, "nmm:focalLength");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.focal_length);
+			g_free (merge_data.focal_length);
+		}
+
+		if (merge_data.metering_mode) {
+			tracker_sparql_builder_predicate (metadata, "nmm:meteringMode");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.metering_mode);
+			g_free (merge_data.metering_mode);
+		}
+
 
 		if (xmp_data.keywords) {
 			insert_keywords (metadata, uri, xmp_data.keywords);
 			g_free (xmp_data.keywords);
 		}
 
+		if (xmp_data.pdf_keywords) {
+			insert_keywords (metadata, uri, xmp_data.pdf_keywords);
+			g_free (xmp_data.pdf_keywords);
+		}
+
+		if (xmp_data.rating) {
+			tracker_sparql_builder_predicate (metadata, "nao:numericRating");
+			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.rating);
+			g_free (xmp_data.rating);
+		}
+
 		if (xmp_data.subject) {
 			insert_keywords (metadata, uri, xmp_data.subject);
 			g_free (xmp_data.subject);
@@ -326,82 +504,46 @@ read_metadata (png_structp png_ptr, png_infop info_ptr, const gchar *uri, Tracke
 			g_free (xmp_data.coverage);
 		}
 
-		if (xmp_data.Make || xmp_data.Model) {
-			gchar *final_camera = tracker_merge (" ", 2, xmp_data.Make, xmp_data.Model);
-
-			tracker_sparql_builder_predicate (metadata, "nmm:camera");
-			tracker_sparql_builder_object_unvalidated (metadata, final_camera);
-			g_free (final_camera);
-		}
-
-		if (xmp_data.Orientation) {
-			tracker_sparql_builder_predicate (metadata, "nfo:orientation");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.Orientation);
-			g_free (xmp_data.Orientation);
-		}
-
-		if (xmp_data.WhiteBalance) {
-			tracker_sparql_builder_predicate (metadata, "nmm:whiteBalance");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.WhiteBalance);
-			g_free (xmp_data.WhiteBalance);
-		}
-
-		if (xmp_data.FNumber) {
-			tracker_sparql_builder_predicate (metadata, "nmm:fnumber");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.FNumber);
-			g_free (xmp_data.FNumber);
-		}
-
-		if (xmp_data.Flash) {
-			tracker_sparql_builder_predicate (metadata, "nmm:flash");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.Flash);
-			g_free (xmp_data.Flash);
-		}
-
-		if (xmp_data.FocalLength) {
-			tracker_sparql_builder_predicate (metadata, "nmm:focalLength");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.FocalLength);
-			g_free (xmp_data.FocalLength);
-		}
-
-		if (xmp_data.Artist || xmp_data.contributor) {
-			gchar *final_artist =  tracker_coalesce (2, xmp_data.Artist, xmp_data.contributor);
-
-			tracker_sparql_builder_predicate (metadata, "nco:contributor");
-
+		if (xmp_data.address || xmp_data.country || xmp_data.city) {
+			tracker_sparql_builder_predicate (metadata, "mlo:location");
+	
 			tracker_sparql_builder_object_blank_open (metadata);
 			tracker_sparql_builder_predicate (metadata, "a");
-			tracker_sparql_builder_object (metadata, "nco:Contact");
-
-			tracker_sparql_builder_predicate (metadata, "nco:fullname");
-			tracker_sparql_builder_object_unvalidated (metadata, final_artist);
+			tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
+	
+			if (xmp_data.address) {
+				tracker_sparql_builder_predicate (metadata, "mlo:address");
+				tracker_sparql_builder_object_unvalidated (metadata, xmp_data.address);
+				g_free (xmp_data.address);
+			}
+	
+			if (xmp_data.state) {
+				tracker_sparql_builder_predicate (metadata, "mlo:state");
+				tracker_sparql_builder_object_unvalidated (metadata, xmp_data.state);
+				g_free (xmp_data.state);
+			}
+	
+			if (xmp_data.city) {
+				tracker_sparql_builder_predicate (metadata, "mlo:city");
+				tracker_sparql_builder_object_unvalidated (metadata, xmp_data.city);
+				g_free (xmp_data.city);
+			}
+	
+			if (xmp_data.country) {
+				tracker_sparql_builder_predicate (metadata, "mlo:country");
+				tracker_sparql_builder_object_unvalidated (metadata, xmp_data.country);
+				g_free (xmp_data.country);
+			}
+		
 			tracker_sparql_builder_object_blank_close (metadata);
-			g_free (final_artist);
-		}
-
-		if (xmp_data.ExposureTime) {
-			tracker_sparql_builder_predicate (metadata, "nmm:exposureTime");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.ExposureTime);
-			g_free (xmp_data.ExposureTime);
-		}
-
-		if (xmp_data.ISOSpeedRatings) {
-			tracker_sparql_builder_predicate (metadata, "nmm:isoSpeed");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.ISOSpeedRatings);
-			g_free (xmp_data.ISOSpeedRatings);
-		}
-
-		if (xmp_data.MeteringMode) {
-			tracker_sparql_builder_predicate (metadata, "nmm:meteringMode");
-			tracker_sparql_builder_object_unvalidated (metadata, xmp_data.MeteringMode);
-			g_free (xmp_data.MeteringMode);
 		}
 	}
 }
 
 static void
-extract_png (const gchar *uri,
-             TrackerSparqlBuilder   *metadata)
+extract_png (const gchar          *uri,
+             TrackerSparqlBuilder *preupdate,
+             TrackerSparqlBuilder *metadata)
 {
 	goffset      size;
 	FILE        *f;
@@ -494,7 +636,6 @@ extract_png (const gchar *uri,
 
 		png_read_end (png_ptr, end_ptr);
 
-		tracker_sparql_builder_subject_iri (metadata, uri);
 		tracker_sparql_builder_predicate (metadata, "a");
 		tracker_sparql_builder_object (metadata, "nfo:Image");
 		tracker_sparql_builder_object (metadata, "nmm:Photo");
@@ -516,7 +657,7 @@ extract_png (const gchar *uri,
 }
 
 TrackerExtractData *
-tracker_get_extract_data (void)
+tracker_extract_get_data (void)
 {
 	return data;
 }
