@@ -20,8 +20,13 @@
 #include "config.h"
 
 #include <sys/statvfs.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <linux/msdos_fs.h>
+#include <unistd.h>
 
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 #include <libtracker-common/tracker-ontology.h>
 #include <libtracker-common/tracker-power.h>
@@ -34,7 +39,6 @@
 #include "tracker-miner-files.h"
 #include "tracker-config.h"
 #include "tracker-extract-client.h"
-#include "tracker-thumbnailer.h"
 #include "tracker-marshal.h"
 
 #define DISK_SPACE_CHECK_FREQUENCY 10
@@ -60,12 +64,18 @@ struct TrackerMinerFilesPrivate {
 
 	GVolumeMonitor *volume_monitor;
 
+        GSList *index_recursive_directories;
+        GSList *index_single_directories;
+
 	guint disk_space_check_id;
 	guint disk_space_pause_cookie;
 
 	guint low_battery_pause_cookie;
 
 	DBusGProxy *extractor_proxy;
+
+	GQuark quark_mount_point_udi;
+	GQuark quark_directory_config_root;
 };
 
 enum {
@@ -78,67 +88,65 @@ enum {
 	PROP_CONFIG
 };
 
-static void     miner_files_set_property      (GObject              *object,
-                                               guint                 param_id,
-                                               const GValue         *value,
-                                               GParamSpec           *pspec);
-static void     miner_files_get_property      (GObject              *object,
-                                               guint                 param_id,
-                                               GValue               *value,
-                                               GParamSpec           *pspec);
-static void     miner_files_finalize          (GObject              *object);
-static void     miner_files_constructed       (GObject              *object);
-
-static void     mount_pre_unmount_cb          (GVolumeMonitor       *volume_monitor,
-                                               GMount               *mount,
-                                               TrackerMinerFiles    *mf);
+static void        miner_files_set_property             (GObject              *object,
+                                                         guint                 param_id,
+                                                         const GValue         *value,
+                                                         GParamSpec           *pspec);
+static void        miner_files_get_property             (GObject              *object,
+                                                         guint                 param_id,
+                                                         GValue               *value,
+                                                         GParamSpec           *pspec);
+static void        miner_files_finalize                 (GObject              *object);
+static void        miner_files_constructed              (GObject              *object);
+static void        mount_pre_unmount_cb                 (GVolumeMonitor       *volume_monitor,
+                                                         GMount               *mount,
+                                                         TrackerMinerFiles    *mf);
 
 #ifdef HAVE_HAL
-static void     mount_point_added_cb          (TrackerStorage       *storage,
-                                               const gchar          *udi,
-                                               const gchar          *mount_point,
-                                               gpointer              user_data);
-static void     mount_point_removed_cb        (TrackerStorage       *storage,
-                                               const gchar          *udi,
-                                               const gchar          *mount_point,
-                                               gpointer              user_data);
-
-static void     check_battery_status          (TrackerMinerFiles    *fs);
-static void     battery_status_cb             (GObject              *object,
-                                               GParamSpec           *pspec,
-                                               gpointer              user_data);
+static void        mount_point_added_cb                 (TrackerStorage       *storage,
+                                                         const gchar          *udi,
+                                                         const gchar          *mount_point,
+                                                         gpointer              user_data);
+static void        mount_point_removed_cb               (TrackerStorage       *storage,
+                                                         const gchar          *udi,
+                                                         const gchar          *mount_point,
+                                                         gpointer              user_data);
+static void        check_battery_status                 (TrackerMinerFiles    *fs);
+static void        battery_status_cb                    (GObject              *object,
+                                                         GParamSpec           *pspec,
+                                                         gpointer              user_data);
 #endif
 
-static void     init_mount_points             (TrackerMinerFiles *miner);
-static void     disk_space_check_start        (TrackerMinerFiles    *mf);
-static void     disk_space_check_stop         (TrackerMinerFiles    *mf);
-static void     low_disk_space_limit_cb       (GObject              *gobject,
-                                               GParamSpec           *arg1,
-                                               gpointer              user_data);
-
-static DBusGProxy * extractor_create_proxy    (void);
-static void    extractor_queue_thumbnail_cb   (DBusGProxy           *proxy,
-                                               const gchar          *filename,
-                                               const gchar          *mime_type,
-                                               gpointer              user_data);
-
-static gboolean miner_files_check_file        (TrackerMinerFS       *fs,
-                                               GFile                *file);
-static gboolean miner_files_check_directory   (TrackerMinerFS       *fs,
-                                               GFile                *file);
-static gboolean miner_files_check_directory_contents (TrackerMinerFS       *fs,
-                                                      GFile                *parent,
-                                                      GList                *children);
-static gboolean miner_files_process_file      (TrackerMinerFS       *fs,
-                                               GFile                *file,
-                                               TrackerSparqlBuilder *sparql,
-                                               GCancellable         *cancellable);
-static gboolean miner_files_monitor_directory (TrackerMinerFS       *fs,
-                                               GFile                *file);
-static gboolean miner_files_writeback_file    (TrackerMinerFS       *fs,
-                                               GFile                *file,
-                                               TrackerSparqlBuilder *sparql,
-                                               GCancellable         *cancellable);
+static void        init_mount_points                    (TrackerMinerFiles    *miner);
+static void        disk_space_check_start               (TrackerMinerFiles    *mf);
+static void        disk_space_check_stop                (TrackerMinerFiles    *mf);
+static void        low_disk_space_limit_cb              (GObject              *gobject,
+                                                         GParamSpec           *arg1,
+                                                         gpointer              user_data);
+static void        index_recursive_directories_cb       (GObject              *gobject,
+                                                         GParamSpec           *arg1,
+                                                         gpointer              user_data);
+static void        index_single_directories_cb          (GObject              *gobject,
+                                                         GParamSpec           *arg1,
+                                                         gpointer              user_data);
+static DBusGProxy *extractor_create_proxy               (void);
+static gboolean    miner_files_check_file               (TrackerMinerFS       *fs,
+                                                         GFile                *file);
+static gboolean    miner_files_check_directory          (TrackerMinerFS       *fs,
+                                                         GFile                *file);
+static gboolean    miner_files_check_directory_contents (TrackerMinerFS       *fs,
+                                                         GFile                *parent,
+                                                         GList                *children);
+static gboolean    miner_files_process_file             (TrackerMinerFS       *fs,
+                                                         GFile                *file,
+                                                         TrackerSparqlBuilder *sparql,
+                                                         GCancellable         *cancellable);
+static gboolean    miner_files_monitor_directory        (TrackerMinerFS       *fs,
+                                                         GFile                *file);
+static gboolean    miner_files_ignore_next_update_file  (TrackerMinerFS       *fs,
+                                                         GFile                *file,
+                                                         TrackerSparqlBuilder *sparql,
+                                                         GCancellable         *cancellable);
 
 G_DEFINE_TYPE (TrackerMinerFiles, tracker_miner_files, TRACKER_TYPE_MINER_FS)
 
@@ -158,7 +166,7 @@ tracker_miner_files_class_init (TrackerMinerFilesClass *klass)
 	miner_fs_class->check_directory_contents = miner_files_check_directory_contents;
 	miner_fs_class->monitor_directory = miner_files_monitor_directory;
 	miner_fs_class->process_file = miner_files_process_file;
-	miner_fs_class->writeback_file = miner_files_writeback_file;
+	miner_fs_class->ignore_next_update_file = miner_files_ignore_next_update_file;
 
 	g_object_class_install_property (object_class,
 	                                 PROP_CONFIG,
@@ -209,20 +217,8 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 	/* Set up extractor and signals */
 	priv->extractor_proxy = extractor_create_proxy ();
 
-	dbus_g_object_register_marshaller (tracker_marshal_VOID__STRING_STRING,
-	                                   G_TYPE_NONE,
-	                                   G_TYPE_STRING,
-	                                   G_TYPE_STRING,
-	                                   G_TYPE_INVALID);
-
-	dbus_g_proxy_add_signal (priv->extractor_proxy, "QueueThumbnail",
-	                         G_TYPE_STRING,
-	                         G_TYPE_STRING,
-	                         G_TYPE_INVALID);
-
-	dbus_g_proxy_connect_signal (priv->extractor_proxy, "QueueThumbnail",
-	                             G_CALLBACK (extractor_queue_thumbnail_cb),
-	                             NULL, NULL);
+	priv->quark_mount_point_udi = g_quark_from_static_string ("tracker-mount-point-udi");
+	priv->quark_directory_config_root = g_quark_from_static_string ("tracker-directory-config-root");
 
 	init_mount_points (mf);
 }
@@ -276,10 +272,6 @@ miner_files_finalize (GObject *object)
 	mf = TRACKER_MINER_FILES (object);
 	priv = mf->private;
 
-	dbus_g_proxy_disconnect_signal (priv->extractor_proxy, "QueueThumbnail",
-	                                G_CALLBACK (extractor_queue_thumbnail_cb),
-	                                NULL);
-
 	g_object_unref (priv->extractor_proxy);
 
 	g_signal_handlers_disconnect_by_func (priv->config,
@@ -290,6 +282,16 @@ miner_files_finalize (GObject *object)
 
 	disk_space_check_stop (TRACKER_MINER_FILES (object));
 
+        if (priv->index_recursive_directories) {
+                g_slist_foreach (priv->index_recursive_directories, (GFunc) g_free, NULL);
+                g_slist_free (priv->index_recursive_directories);
+        }
+
+        if (priv->index_single_directories) {
+                g_slist_foreach (priv->index_single_directories, (GFunc) g_free, NULL);
+                g_slist_free (priv->index_single_directories);
+        }
+
 #ifdef HAVE_HAL
 	g_object_unref (priv->power);
 	g_object_unref (priv->storage);
@@ -334,6 +336,9 @@ miner_files_constructed (GObject *object)
 	/* Fill in directories to inspect */
 	dirs = tracker_config_get_index_single_directories (mf->private->config);
 
+        /* Copy in case of config changes */
+        mf->private->index_single_directories = tracker_gslist_copy_with_string_data (dirs);
+
 	for (; dirs; dirs = dirs->next) {
 		GFile *file;
 
@@ -365,8 +370,13 @@ miner_files_constructed (GObject *object)
 		}
 
 		g_message ("  Adding:'%s'", (gchar*) dirs->data);
+
 		file = g_file_new_for_path (dirs->data);
-		tracker_miner_fs_add_directory (fs, file, FALSE);
+		g_object_set_qdata (G_OBJECT (file),
+		                    mf->private->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
+
+		tracker_miner_fs_directory_add (fs, file, FALSE);
 		g_object_unref (file);
 	}
 
@@ -374,6 +384,9 @@ miner_files_constructed (GObject *object)
 
 	dirs = tracker_config_get_index_recursive_directories (mf->private->config);
 
+        /* Copy in case of config changes */
+        mf->private->index_recursive_directories = tracker_gslist_copy_with_string_data (dirs);
+
 	for (; dirs; dirs = dirs->next) {
 		GFile *file;
 
@@ -405,8 +418,13 @@ miner_files_constructed (GObject *object)
 		}
 
 		g_message ("  Adding:'%s'", (gchar*) dirs->data);
+
 		file = g_file_new_for_path (dirs->data);
-		tracker_miner_fs_add_directory (fs, file, TRUE);
+		g_object_set_qdata (G_OBJECT (file),
+		                    mf->private->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
+
+		tracker_miner_fs_directory_add (fs, file, TRUE);
 		g_object_unref (file);
 	}
 
@@ -415,9 +433,21 @@ miner_files_constructed (GObject *object)
 
 	for (m = mounts; m; m = m->next) {
 		GFile *file = g_file_new_for_path (m->data);
+#ifdef HAVE_HAL
+		const gchar *udi = tracker_storage_get_volume_udi_for_file (mf->private->storage, file);
+
+		g_object_set_qdata_full (G_OBJECT (file),
+					 mf->private->quark_mount_point_udi,
+					 g_strdup (udi),
+					 (GDestroyNotify) g_free);
+#endif
+
+		g_object_set_qdata (G_OBJECT (file),
+		                    mf->private->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
 
 		g_message ("  Adding:'%s'", (gchar*) m->data);
-		tracker_miner_fs_add_directory (TRACKER_MINER_FS (mf),
+		tracker_miner_fs_directory_add (TRACKER_MINER_FS (mf),
 		                                file,
 		                                TRUE);
 		g_object_unref (file);
@@ -426,6 +456,12 @@ miner_files_constructed (GObject *object)
 	g_signal_connect (mf->private->config, "notify::low-disk-space-limit",
 	                  G_CALLBACK (low_disk_space_limit_cb),
 	                  mf);
+	g_signal_connect (mf->private->config, "notify::index-recursive-directories",
+	                  G_CALLBACK (index_recursive_directories_cb),
+	                  mf);
+	g_signal_connect (mf->private->config, "notify::index-single-directories",
+	                  G_CALLBACK (index_single_directories_cb),
+	                  mf);
 
 	g_slist_foreach (mounts, (GFunc) g_free, NULL);
 	g_slist_free (mounts);
@@ -476,9 +512,15 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			uri = g_file_get_uri (file);
 
 			g_string_append_printf (queries,
-			                        "DROP GRAPH <%s>\n "
-			                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:mountPoint <%s> } ",
-			                        removable_device_urn, removable_device_urn, removable_device_urn, uri);
+			                        "DROP GRAPH <%s> "
+			                        "INSERT INTO <%s> { "
+			                        "  <%s> a tracker:Volume; "
+			                        "       tracker:mountPoint ?u "
+			                        "} WHERE { "
+			                        "  ?u a nfo:FileDataObject; "
+			                        "     nie:url \"%s\" "
+			                        "}",
+						removable_device_urn, removable_device_urn, removable_device_urn, uri);
 
 			g_object_unref (file);
 			g_free (uri);
@@ -728,7 +770,16 @@ mount_point_added_cb (TrackerStorage *storage,
 		GFile *file;
 
 		file = g_file_new_for_path (mount_point);
-		tracker_miner_fs_add_directory (TRACKER_MINER_FS (user_data),
+		g_object_set_qdata_full (G_OBJECT (file),
+		                         priv->quark_mount_point_udi,
+		                         g_strdup (udi),
+		                         (GDestroyNotify) g_free);
+
+		g_object_set_qdata (G_OBJECT (file),
+		                    priv->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
+
+		tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
 		                                file,
 		                                TRUE);
 		g_object_unref (file);
@@ -828,7 +879,7 @@ mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
 	GFile *mount_root;
 
 	mount_root = g_mount_get_root (mount);
-	tracker_miner_fs_remove_directory (TRACKER_MINER_FS (mf), mount_root);
+	tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf), mount_root);
 	g_object_unref (mount_root);
 }
 
@@ -939,6 +990,115 @@ low_disk_space_limit_cb (GObject    *gobject,
 	disk_space_check_cb (mf);
 }
 
+static void
+update_directories_from_new_config (TrackerMinerFS *mf,
+                                    GSList         *new_dirs,
+                                    GSList         *old_dirs,
+                                    gboolean        recurse)
+{
+	TrackerMinerFilesPrivate *priv;
+        GSList *sl;
+
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (mf);
+
+        g_message ("Updating %s directories changed from configuration",
+                   recurse ? "recursive" : "single");
+
+        /* First remove all directories removed from the config */
+        for (sl = old_dirs; sl; sl = sl->next) {
+                const gchar *path;
+
+                path = sl->data;
+
+                /* If we are not still in the list, remove the dir */
+                if (!tracker_string_in_gslist (path, new_dirs)) {
+                        GFile *file;
+
+                        g_message ("  Removing directory:'%s'", path);
+
+                        file = g_file_new_for_path (path);
+                        tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf), file);
+                        g_object_unref (file);
+                }
+        }
+
+        /* Second add directories which are new */
+        for (sl = new_dirs; sl; sl = sl->next) {
+                const gchar *path;
+
+                path = sl->data;
+
+                /* If we are now in the list, add the dir */
+                if (!tracker_string_in_gslist (path, old_dirs)) {
+                        GFile *file;
+
+                        g_message ("  Adding directory:'%s'", path);
+
+                        file = g_file_new_for_path (path);
+			g_object_set_qdata (G_OBJECT (file),
+			                    priv->quark_directory_config_root,
+			                    GINT_TO_POINTER (TRUE));
+
+			tracker_miner_fs_directory_add (TRACKER_MINER_FS (mf), file, recurse);
+                        g_object_unref (file);
+                }
+        }
+}
+
+static void
+index_recursive_directories_cb (GObject    *gobject,
+                                GParamSpec *arg1,
+                                gpointer    user_data)
+{
+        TrackerMinerFilesPrivate *private;
+        GSList *new_dirs, *old_dirs;
+
+        private = TRACKER_MINER_FILES_GET_PRIVATE (user_data);
+
+        new_dirs = tracker_config_get_index_recursive_directories (private->config);
+        old_dirs = private->index_recursive_directories;
+
+        update_directories_from_new_config (TRACKER_MINER_FS (user_data),
+                                            new_dirs,
+                                            old_dirs,
+                                            TRUE);
+
+        /* Re-set the stored config in case it changes again */
+        if (private->index_recursive_directories) {
+                g_slist_foreach (private->index_recursive_directories, (GFunc) g_free, NULL);
+                g_slist_free (private->index_recursive_directories);
+        }
+
+        private->index_recursive_directories = tracker_gslist_copy_with_string_data (new_dirs);
+}
+
+static void
+index_single_directories_cb (GObject    *gobject,
+                             GParamSpec *arg1,
+                             gpointer    user_data)
+{
+        TrackerMinerFilesPrivate *private;
+        GSList *new_dirs, *old_dirs;
+
+        private = TRACKER_MINER_FILES_GET_PRIVATE (user_data);
+
+        new_dirs = tracker_config_get_index_single_directories (private->config);
+        old_dirs = private->index_single_directories;
+
+        update_directories_from_new_config (TRACKER_MINER_FS (user_data),
+                                            new_dirs,
+                                            old_dirs,
+                                            FALSE);
+
+        /* Re-set the stored config in case it changes again */
+        if (private->index_single_directories) {
+                g_slist_foreach (private->index_single_directories, (GFunc) g_free, NULL);
+                g_slist_free (private->index_single_directories);
+        }
+
+        private->index_single_directories = tracker_gslist_copy_with_string_data (new_dirs);
+}
+
 static gboolean
 miner_files_check_file (TrackerMinerFS *fs,
                         GFile          *file)
@@ -986,7 +1146,7 @@ miner_files_check_file (TrackerMinerFS *fs,
 
 	should_process = TRUE;
 
- done:
+done:
 	g_free (basename);
 	g_free (path);
 
@@ -1007,6 +1167,7 @@ miner_files_check_directory (TrackerMinerFS *fs,
 	gchar *basename;
 	gchar *path;
 	gboolean should_process;
+	gboolean is_hidden;
 
 	should_process = FALSE;
 	basename = NULL;
@@ -1019,7 +1180,32 @@ miner_files_check_directory (TrackerMinerFS *fs,
 
 	path = g_file_get_path (file);
 
-	if (file_info && g_file_info_get_is_hidden (file_info)) {
+	/* First we check the GIO hidden check. This does a number of
+	 * things for us which is good (like checking ".foo" dirs).
+	 */
+	is_hidden = file_info && g_file_info_get_is_hidden (file_info);
+
+	/* Second we check if the file is on FAT and if the hidden
+	 * attribute is set. GIO does this but ONLY on a Windows OS,
+	 * not for Windows files under a Linux OS, so we have to check
+	 * anyway.
+	 */
+	if (!is_hidden) {
+		int fd;
+
+		fd = g_open (path, O_RDONLY, 0);
+		if (fd != -1) {
+			__u32 attrs;
+
+			if (ioctl (fd, FAT_IOCTL_GET_ATTRIBUTES, &attrs) == 0) {
+				is_hidden = attrs & ATTR_HIDDEN ? TRUE : FALSE;
+			}
+
+			close (fd);
+		}
+	}
+
+	if (is_hidden) {
 		TrackerMinerFiles *mf;
 		GSList *allowed_directories;
 
@@ -1162,7 +1348,7 @@ miner_files_add_to_datasource (TrackerMinerFiles    *mf,
 		removable_device_urn = g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN);
 	}
 
-	tracker_sparql_builder_subject_iri (sparql, uri);
+	tracker_sparql_builder_subject (sparql, "_:file");
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
 
@@ -1215,39 +1401,107 @@ extractor_create_proxy (void)
 	return proxy;
 }
 
-static void
-extractor_queue_thumbnail_cb (DBusGProxy  *proxy,
-                              const gchar *filename,
-                              const gchar *mime_type,
-                              gpointer     user_data)
-{
-	tracker_thumbnailer_queue_add (filename, mime_type);
-}
-
 static void
 extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
+                                    gchar      *preupdate,
                                     gchar      *sparql,
                                     GError     *error,
                                     gpointer    user_data)
 {
+	TrackerMinerFilesPrivate *priv;
 	ProcessFileData *data = user_data;
+	const gchar *udi;
+
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (data->miner);
 
 	if (error) {
 		/* Something bad happened, notify about the error */
-		tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, error);
+		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
 		process_file_data_free (data);
 		g_error_free (error);
 		return;
 	}
 
-	if (sparql) {
+	if (sparql && *sparql) {
+		tracker_sparql_builder_append (data->sparql, "\n");
 		tracker_sparql_builder_append (data->sparql, sparql);
-		g_free (sparql);
+	}
+
+	tracker_sparql_builder_insert_close (data->sparql);
+
+	if (g_object_get_qdata (G_OBJECT (data->file),
+	                        priv->quark_directory_config_root) == NULL) {
+		GFile *parent;
+
+		parent = g_file_get_parent (data->file);
+
+		if (parent) {
+			gchar *parent_uri;
+
+			parent_uri = g_file_get_uri (parent);
+
+			/* Add where clause for the nfo:belongsToContainer */
+			tracker_sparql_builder_where_open (data->sparql);
+
+			tracker_sparql_builder_subject_variable (data->sparql, "parent");
+			tracker_sparql_builder_predicate (data->sparql, "nie:url");
+			tracker_sparql_builder_object_string (data->sparql, parent_uri);
+
+			tracker_sparql_builder_where_close (data->sparql);
+
+			g_free (parent_uri);
+			g_object_unref (parent);
+		}
+	}
+
+	/* Prepend preupdate queries */
+	if (preupdate && *preupdate) {
+		tracker_sparql_builder_prepend (data->sparql, preupdate);
+	}
+
+	udi = g_object_get_qdata (G_OBJECT (data->file),
+				  data->miner->private->quark_mount_point_udi);
+
+	/* File represents a mount point */
+	if (G_UNLIKELY (udi)) {
+		GString *queries;
+		gchar *removable_device_urn, *uri;
+
+		removable_device_urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", udi);
+		uri = g_file_get_uri (G_FILE (data->file));
+
+		queries = g_string_new ("");
+		g_string_append_printf (queries,
+		                        "DELETE FROM <%s> { "
+		                        "  <%s> tracker:mountPoint ?unknown "
+		                        "} WHERE { "
+		                        "  <%s> a tracker:Volume; "
+		                        "       tracker:mountPoint ?unknown "
+		                        "} ",
+		                        removable_device_urn, removable_device_urn, removable_device_urn);
+
+		g_string_append_printf (queries,
+		                        "INSERT INTO <%s> { "
+		                        "  <%s> a tracker:Volume; "
+		                        "       tracker:mountPoint ?u "
+		                        "} WHERE { "
+		                        "  ?u a nfo:FileDataObject; "
+		                        "     nie:url \"%s\" "
+		                        "}",
+		                        removable_device_urn, removable_device_urn, uri);
+
+		tracker_sparql_builder_append (data->sparql, queries->str);
+		g_string_free (queries, TRUE);
+		g_free (removable_device_urn);
+		g_free (uri);
 	}
 
 	/* Notify about the success */
-	tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, NULL);
+	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, NULL);
+
 	process_file_data_free (data);
+	g_free (preupdate);
+	g_free (sparql);
 }
 
 static void
@@ -1261,7 +1515,7 @@ extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
 	                          data->call);
 
 	error = g_error_new_literal (miner_files_error_quark, 0, "Embedded metadata extraction was cancelled");
-	tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, error);
+	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
 
 	process_file_data_free (data);
 	g_error_free (error);
@@ -1286,23 +1540,25 @@ process_file_cb (GObject      *object,
                  GAsyncResult *result,
                  gpointer      user_data)
 {
+	TrackerMinerFilesPrivate *priv;
 	TrackerSparqlBuilder *sparql;
 	ProcessFileData *data;
 	const gchar *mime_type;
 	GFileInfo *file_info;
 	guint64 time_;
 	GFile *file, *parent;
-	gchar *uri, *parent_uri;
+	gchar *uri;
 	GError *error = NULL;
 
 	data = user_data;
 	file = G_FILE (object);
 	sparql = data->sparql;
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (data->miner);
 	file_info = g_file_query_info_finish (file, result, &error);
 
 	if (error) {
 		/* Something bad happened, notify about the error */
-		tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), file, error);
+		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), file, error);
 		process_file_data_free (data);
 		g_error_free (error);
 		return;
@@ -1313,17 +1569,25 @@ process_file_cb (GObject      *object,
 
 	tracker_sparql_builder_insert_open (sparql, uri);
 
-	tracker_sparql_builder_subject_iri (sparql, uri);
+	tracker_sparql_builder_subject (sparql, "_:file");
+
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
+	tracker_sparql_builder_object (sparql, "nie:InformationElement");
 
-	parent = g_file_get_parent (file);
-	if (parent) {
-		parent_uri = g_file_get_uri (parent);
-		tracker_sparql_builder_predicate (sparql, "nfo:belongsToContainer");
-		tracker_sparql_builder_object_iri (sparql, parent_uri);
-		g_free (parent_uri);
-		g_object_unref (parent);
+	if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
+		tracker_sparql_builder_object (sparql, "nfo:Folder");
+	}
+
+	if (g_object_get_qdata (G_OBJECT (data->file),
+	                        priv->quark_directory_config_root) == NULL) {
+		parent = g_file_get_parent (file);
+
+		if (parent) {
+			tracker_sparql_builder_predicate (sparql, "nfo:belongsToContainer");
+			tracker_sparql_builder_object_variable (sparql, "parent");
+			g_object_unref (parent);
+		}
 	}
 
 	tracker_sparql_builder_predicate (sparql, "nfo:fileName");
@@ -1340,17 +1604,9 @@ process_file_cb (GObject      *object,
 	tracker_sparql_builder_predicate (sparql, "nfo:fileLastAccessed");
 	tracker_sparql_builder_object_date (sparql, (time_t *) &time_);
 
-	tracker_sparql_builder_subject_iri (sparql, uri); /* Change to URN */
-	tracker_sparql_builder_predicate (sparql, "a");
-	tracker_sparql_builder_object (sparql, "nie:InformationElement");
-
-	if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
-		tracker_sparql_builder_object (sparql, "nfo:Folder");
-	}
-
 	/* Laying the link between the IE and the DO. We use IE = DO */
 	tracker_sparql_builder_predicate (sparql, "nie:isStoredAs");
-	tracker_sparql_builder_object_iri (sparql, uri);
+	tracker_sparql_builder_object (sparql, "_:file");
 
 	/* The URL of the DataObject (because IE = DO, this is correct) */
 	tracker_sparql_builder_predicate (sparql, "nie:url");
@@ -1361,13 +1617,6 @@ process_file_cb (GObject      *object,
 
 	miner_files_add_to_datasource (data->miner, file, sparql);
 
-	tracker_sparql_builder_insert_close (sparql);
-
-	/* Send file/mime data to thumbnailer (which adds it to the
-	 * queue if the thumbnailer handles those mime types).
-	 */
-	tracker_thumbnailer_queue_add (uri, mime_type);
-
 	/* Next step, getting embedded metadata */
 	extractor_get_embedded_metadata (data, uri, mime_type);
 
@@ -1409,10 +1658,10 @@ miner_files_process_file (TrackerMinerFS       *fs,
 }
 
 static gboolean
-miner_files_writeback_file (TrackerMinerFS       *fs,
-                            GFile                *file,
-                            TrackerSparqlBuilder *sparql,
-                            GCancellable         *cancellable)
+miner_files_ignore_next_update_file (TrackerMinerFS       *fs,
+                                     GFile                *file,
+                                     TrackerSparqlBuilder *sparql,
+                                     GCancellable         *cancellable)
 {
 	const gchar *attrs;
 	const gchar *mime_type;
@@ -1433,7 +1682,7 @@ miner_files_writeback_file (TrackerMinerFS       *fs,
 	                               cancellable, &error);
 
 	if (error) {
-		g_warning ("Can't writeback: '%s'", error->message);
+		g_warning ("Can't ignore-next-update: '%s'", error->message);
 		g_clear_error (&error);
 		return FALSE;
 	}
@@ -1441,12 +1690,13 @@ miner_files_writeback_file (TrackerMinerFS       *fs,
 	uri = g_file_get_uri (file);
 	mime_type = g_file_info_get_content_type (file_info);
 
-	/* For writeback we only write a few properties back. These properties
-	 * should NEVER be marked as tracker:writeback in the ontology ! */
+	/* For ignore-next-update we only write a few properties back. These properties
+	 * should NEVER be marked as tracker:writeback in the ontology! (else you break
+	 * the tracker-writeback feature) */
 
 	tracker_sparql_builder_insert_open (sparql, uri);
 
-	tracker_sparql_builder_subject_iri (sparql, uri);
+	tracker_sparql_builder_subject_variable (sparql, "urn");
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
 
@@ -1464,11 +1714,19 @@ miner_files_writeback_file (TrackerMinerFS       *fs,
 	tracker_sparql_builder_predicate (sparql, "nie:mimeType");
 	tracker_sparql_builder_object_string (sparql, mime_type);
 
+	tracker_sparql_builder_insert_close (sparql);
+
+	tracker_sparql_builder_where_open (sparql);
+
+	tracker_sparql_builder_subject_variable (sparql, "urn");
+	tracker_sparql_builder_predicate (sparql, "nie:url");
+	tracker_sparql_builder_object_string (sparql, uri);
+
+	tracker_sparql_builder_where_close (sparql);
+
 	g_object_unref (file_info);
 	g_free (uri);
 
-	tracker_sparql_builder_insert_close (sparql);
-
 	return TRUE;
 }
 
