@@ -167,7 +167,13 @@ get_files_foreach (gpointer value,
 {
 	gchar **data = value;
 
-	g_print ("  %s\n", data[0]);
+	g_print ("  %s", data[0]);
+
+	if (data[1] && *data[1]) {
+		g_print (" (URI: %s)", data[1]);
+	}
+
+	g_print ("\n");
 }
 
 static gboolean
@@ -229,10 +235,10 @@ get_document_files (TrackerClient *client,
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT ?document "
+		query = g_strdup_printf ("SELECT ?document nie:url(?document) "
 		                         "WHERE { "
 		                         "  ?document a nfo:Document ."
-		                         "  ?document fts:match \"%s\" "
+		                         "  ?document fts:match \"%s\" . "
 		                         "} "
 		                         "ORDER BY ASC(?document) "
 		                         "OFFSET %d "
@@ -241,9 +247,9 @@ get_document_files (TrackerClient *client,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT ?document "
+		query = g_strdup_printf ("SELECT ?document nie:url(?document) "
 		                         "WHERE { "
-		                         "  ?document a nfo:Document "
+		                         "  ?document a nfo:Document . "
 		                         "} "
 		                         "ORDER BY ASC(?document) "
 		                         "OFFSET %d "
@@ -273,10 +279,10 @@ get_video_files (TrackerClient *client,
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT ?video "
+		query = g_strdup_printf ("SELECT ?video nie:url(?video) "
 		                         "WHERE { "
 		                         "  ?video a nfo:Video ."
-		                         "  ?video fts:match \"%s\" "
+		                         "  ?video fts:match \"%s\" . "
 		                         "} "
 		                         "ORDER BY ASC(?video) "
 		                         "OFFSET %d "
@@ -285,9 +291,9 @@ get_video_files (TrackerClient *client,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT ?video "
+		query = g_strdup_printf ("SELECT ?video nie:url(?video) "
 		                         "WHERE { "
-		                         "  ?video a nfo:Video "
+		                         "  ?video a nfo:Video . "
 		                         "} "
 		                         "ORDER BY ASC(?video) "
 		                         "OFFSET %d "
@@ -317,10 +323,10 @@ get_image_files (TrackerClient *client,
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT ?image "
+		query = g_strdup_printf ("SELECT ?image nie:url(?image) "
 		                         "WHERE { "
 		                         "  ?image a nfo:Image ."
-		                         "  ?image fts:match \"%s\" "
+		                         "  ?image fts:match \"%s\" . "
 		                         "} "
 		                         "ORDER BY ASC(?image) "
 		                         "OFFSET %d "
@@ -329,9 +335,9 @@ get_image_files (TrackerClient *client,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT ?image "
+		query = g_strdup_printf ("SELECT ?image nie:url(?image) "
 		                         "WHERE { "
-		                         "  ?image a nfo:Image "
+		                         "  ?image a nfo:Image . "
 		                         "} "
 		                         "ORDER BY ASC(?image) "
 		                         "OFFSET %d "
@@ -361,10 +367,10 @@ get_music_files (TrackerClient *client,
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT ?song "
+		query = g_strdup_printf ("SELECT ?song nie:url(?song) "
 		                         "WHERE { "
-		                         "  ?song a nmm:MusicPiece ."
-		                         "  ?song fts:match \"%s\" "
+		                         "  ?song a nmm:MusicPiece . "
+		                         "  ?song fts:match \"%s\" . "
 		                         "} "
 		                         "ORDER BY ASC(?song) "
 		                         "OFFSET %d "
@@ -373,9 +379,9 @@ get_music_files (TrackerClient *client,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT ?song "
+		query = g_strdup_printf ("SELECT ?song nie:url(?song) "
 		                         "WHERE { "
-		                         "  ?song a nmm:MusicPiece "
+		                         "  ?song a nmm:MusicPiece . "
 		                         "} "
 		                         "ORDER BY ASC(?song) "
 		                         "OFFSET %d "
@@ -520,7 +526,7 @@ get_music_albums (TrackerClient *client,
 	if (fts) {
 		query = g_strdup_printf ("SELECT ?album nie:title (?album) COUNT(?song) "
 		                         "AS songs "
-		                         "SUM(nmm:length (?length)) AS totallength "
+		                         "SUM(nfo:duration (?song)) AS totallength "
 		                         "WHERE {"
 		                         "  ?album a nmm:MusicAlbum ."
 		                         "  ?song nmm:musicAlbum ?album ."
@@ -535,7 +541,7 @@ get_music_albums (TrackerClient *client,
 	} else {
 		query = g_strdup_printf ("SELECT ?album nie:title (?album) COUNT(?song) "
 		                         "AS songs "
-		                         "SUM(nmm:length (?length)) AS totallength "
+		                         "SUM(nfo:duration (?song)) AS totallength "
 		                         "WHERE {"
 		                         "  ?album a nmm:MusicAlbum ."
 		                         "  ?song nmm:musicAlbum ?album ." 
@@ -601,10 +607,10 @@ get_files (TrackerClient *client,
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT ?u "
+		query = g_strdup_printf ("SELECT ?u nie:url(?u) "
 		                         "WHERE { "
 		                         "  ?u a nie:InformationElement ."
-		                         "  ?u fts:match \"%s\" "
+		                         "  ?u fts:match \"%s\" . "
 		                         "} "
 		                         "ORDER BY ASC(?u) "
 		                         "OFFSET %d "
@@ -613,9 +619,9 @@ get_files (TrackerClient *client,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT ?u "
+		query = g_strdup_printf ("SELECT ?u nie:url(?u) "
 		                         "WHERE { "
-		                         "  ?u a nie:InformationElement "
+		                         "  ?u a nie:InformationElement . "
 		                         "} "
 		                         "ORDER BY ASC(?u) "
 		                         "OFFSET %d "
@@ -645,7 +651,7 @@ get_folders (TrackerClient *client,
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT ?u "
+		query = g_strdup_printf ("SELECT ?u nie:url(?u) "
 		                         "WHERE { "
 		                         "  ?u a nfo:Folder ."
 		                         "  ?u fts:match \"%s\" "
@@ -657,9 +663,9 @@ get_folders (TrackerClient *client,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT ?u "
+		query = g_strdup_printf ("SELECT ?u nie:url(?u) "
 		                         "WHERE { "
-		                         "  ?u a nfo:Folder "
+		                         "  ?u a nfo:Folder . "
 		                         "} "
 		                         "ORDER BY ASC(?u) "
 		                         "OFFSET %d "
